package cn.com.infosec.math.ec;

import java.math.BigInteger;

class SimpleBigDecimal {
   private static final long serialVersionUID = 1L;
   private final BigInteger bigInt;
   private final int scale;

   public static SimpleBigDecimal getInstance(BigInteger value, int scale) {
      return new SimpleBigDecimal(value.shiftLeft(scale), scale);
   }

   public SimpleBigDecimal(BigInteger bigInt, int scale) {
      if (scale >= 0) {
         this.bigInt = bigInt;
         this.scale = scale;
      } else {
         throw new IllegalArgumentException("scale may not be negative");
      }
   }

   private SimpleBigDecimal(SimpleBigDecimal limBigDec) {
      this.bigInt = limBigDec.bigInt;
      this.scale = limBigDec.scale;
   }

   private void checkScale(SimpleBigDecimal b) {
      if (this.scale != b.scale) {
         throw new IllegalArgumentException("Only SimpleBigDecimal of same scale allowed in arithmetic operations");
      }
   }

   public SimpleBigDecimal adjustScale(int newScale) {
      if (newScale >= 0) {
         int var2;
         return newScale == (var2 = this.scale) ? new SimpleBigDecimal(this) : new SimpleBigDecimal(this.bigInt.shiftLeft(newScale - var2), newScale);
      } else {
         throw new IllegalArgumentException("scale may not be negative");
      }
   }

   public SimpleBigDecimal add(SimpleBigDecimal b) {
      this.checkScale(b);
      SimpleBigDecimal var10002 = this;
      BigInteger var2 = this.bigInt.add(b.bigInt);
      return new SimpleBigDecimal(var2, var10002.scale);
   }

   public SimpleBigDecimal add(BigInteger b) {
      SimpleBigDecimal var10002 = this;
      BigInteger var2 = this.bigInt.add(b.shiftLeft(this.scale));
      return new SimpleBigDecimal(var2, var10002.scale);
   }

   public SimpleBigDecimal negate() {
      SimpleBigDecimal var10002 = this;
      BigInteger var1 = this.bigInt.negate();
      return new SimpleBigDecimal(var1, var10002.scale);
   }

   public SimpleBigDecimal subtract(SimpleBigDecimal b) {
      return this.add(b.negate());
   }

   public SimpleBigDecimal subtract(BigInteger b) {
      SimpleBigDecimal var10002 = this;
      BigInteger var2 = this.bigInt.subtract(b.shiftLeft(this.scale));
      return new SimpleBigDecimal(var2, var10002.scale);
   }

   public SimpleBigDecimal multiply(SimpleBigDecimal b) {
      this.checkScale(b);
      SimpleBigDecimal var10002 = this;
      BigInteger var2 = this.bigInt.multiply(b.bigInt);
      int var10003 = var10002.scale;
      return new SimpleBigDecimal(var2, var10003 + var10003);
   }

   public SimpleBigDecimal multiply(BigInteger b) {
      SimpleBigDecimal var10002 = this;
      BigInteger var2 = this.bigInt.multiply(b);
      return new SimpleBigDecimal(var2, var10002.scale);
   }

   public SimpleBigDecimal divide(SimpleBigDecimal b) {
      this.checkScale(b);
      BigInteger var2 = this.bigInt.shiftLeft(this.scale);
      SimpleBigDecimal var10002 = this;
      BigInteger var3 = var2.divide(b.bigInt);
      return new SimpleBigDecimal(var3, var10002.scale);
   }

   public SimpleBigDecimal divide(BigInteger b) {
      SimpleBigDecimal var10002 = this;
      BigInteger var2 = this.bigInt.divide(b);
      return new SimpleBigDecimal(var2, var10002.scale);
   }

   public SimpleBigDecimal shiftLeft(int n) {
      SimpleBigDecimal var10002 = this;
      BigInteger var2 = this.bigInt.shiftLeft(n);
      return new SimpleBigDecimal(var2, var10002.scale);
   }

   public int compareTo(SimpleBigDecimal val) {
      this.checkScale(val);
      return this.bigInt.compareTo(val.bigInt);
   }

   public int compareTo(BigInteger val) {
      return this.bigInt.compareTo(val.shiftLeft(this.scale));
   }

   public BigInteger floor() {
      return this.bigInt.shiftRight(this.scale);
   }

   public BigInteger round() {
      return this.add((new SimpleBigDecimal(ECConstants.ONE, 1)).adjustScale(this.scale)).floor();
   }

   public int intValue() {
      return this.floor().intValue();
   }

   public long longValue() {
      return this.floor().longValue();
   }

   public int getScale() {
      return this.scale;
   }

   public String toString() {
      if (this.scale == 0) {
         return this.bigInt.toString();
      } else {
         BigInteger var1 = this.floor();
         BigInteger var2 = this.bigInt.subtract(var1.shiftLeft(this.scale));
         if (this.bigInt.signum() == -1) {
            var2 = ECConstants.ONE.shiftLeft(this.scale).subtract(var2);
         }

         if (var1.signum() == -1 && !var2.equals(ECConstants.ZERO)) {
            var1 = var1.add(ECConstants.ONE);
         }

         String var7 = var1.toString();
         char[] var8 = new char[this.scale];
         String var9;
         int var3 = (var9 = var2.toString(2)).length();
         int var4 = this.scale - var3;

         int var5;
         for(var5 = 0; var5 < var4; ++var5) {
            var8[var5] = '0';
         }

         for(var5 = 0; var5 < var3; ++var5) {
            int var6 = var4 + var5;
            var8[var6] = var9.charAt(var5);
         }

         var9 = new String.<init>(var8);
         StringBuffer var10 = new StringBuffer(var7);
         var10.append(".");
         var10.append(var9);
         return var10.toString();
      }
   }

   public boolean equals(Object o) {
      if (this == o) {
         return true;
      } else if (!(o instanceof SimpleBigDecimal)) {
         return false;
      } else {
         SimpleBigDecimal o1 = (SimpleBigDecimal)o;
         return this.bigInt.equals(o1.bigInt) && this.scale == o1.scale;
      }
   }

   public int hashCode() {
      return this.bigInt.hashCode() ^ this.scale;
   }
}
